# 2D/3D tissue - fracture generation - detailed time evolution

from esys.lsm import *
from esys.lsm.util import *
from esys.lsm.geometry import *
from WallLoader import WallLoaderRunnable




#Parametres of the simulation
#-----------------------------------------------

maxR= 0.6 	#max radius of the particle
minR= 0.2	#min radius of the particle
LX=   80		# sample width
LY=   25		# sample radius
LZ=   5		# sample radius
den=  2.5e-4	# density (previou25 e-4)

Vlt= 50	# top wall velocity
Vlb= 0		# bottom wal velocity

Nsym=  100000000 	#number of time steps
dt=   1e-7 	#time step 

cp_step= 1000	#time step for checkpointers
fs_step= 100	#time step for field savers	  
ramp_time= 500	# ramp time

#Parametres of elastic-Nrot bonds
nK=  10000000.0	# 
nP=  5000000000.0	#
bd=  1	#
pw= 2		# 

nx= 6	    #number of processors - x direction
ny= 1	    #number of processors - y direction
nz= 1	    #number of processors - z direction

# ----------------------------------------------


#instantiate a simulation object:
sim = LsmMpi (numWorkerProcesses = nx*ny*nz, mpiDimList = [nx,ny,nz])

#initialise the neighbour search algorithm:
sim.initNeighbourSearch (
particleType = "NRotSphere",
gridSpacing = 2*maxR+0.2*minR,
verletDist = 0.2*minR
)

# simulation boundaries
domain = BoundingBox ( Vec3 (-9,-9,-9), Vec3 (LX+9,LY*6,LZ+9) )
sim.setSpatialDomain (bBox = domain,circDimList = [False, False, False])

#set timesteping and sample geometry 
sim.setNumTimeSteps (Nsym)
sim.setTimeStepSize (dt)
  # sim.force2dComputations (True)
sim.readGeometry("../GEOM/B_80_02_06.geo");

#density
sim.setParticleDensity(
tag=1,
mask=-1,
Density=den
)

#density
sim.setParticleDensity(
tag=2,
mask=-1,
Density=den
)

#density
sim.setParticleDensity(
tag=3,
mask=-1,
Density=den
)


# ----------- Particles -----------


# bonded particles interaction
bonded = sim.createInteractionGroup(
NRotBondPrms(
 name = "pp_bonds",
 normalK = nK,
 breakDistance = bd,
 tag = 1,
 scaling = True
)
)

#unbonded particles with friction only
unbonded =sim.createInteractionGroup (
NRotFrictionPrms (
	name = "pp_friction",
	normalK = 1000,
	dynamicMu = 0.6,
	shearK = 100.0,
	scaling = True)
)


#create an exclusion between bonded and frictional interactions:
sim.createExclusion (
interactionName1 = "pp_bonds",
interactionName2 = "pp_friction"
)


#add translational viscous damping:
sim.createInteractionGroup (
LinDampingPrms(
name="damping1",
viscosity=0.02,
maxIterations=100
)
)


# ---------- Walls -----------

#create a wall at the bottom of the model:
sim.createWall (
name = "bottom_wall",
posn = Vec3(0.0000, 0.0, 0.0000),
normal = Vec3(0.0000, 1.0000, 0.0000)
)

#create a wall at the top of the model:
sim.createWall (
name = "top_wall",
posn = Vec3(0.0, LY, 0.0000),
normal = Vec3(0.0000, -1.0000, 0.0000)
)

#add a wall loader to move the top wall:
wall_loader1 = WallLoaderRunnable(
LsmMpi = sim,
wallName = "top_wall",
vPlate = Vec3 (0.0, Vlt, 0.0),
startTime = 0,
rampTime = ramp_time
)
sim.addPreTimeStepRunnable (wall_loader1)

#add a wall loader to move the bottom wall:
wall_loader2 = WallLoaderRunnable(
LsmMpi = sim,
wallName = "bottom_wall",
vPlate = Vec3 (0.0, Vlb, 0.0),
startTime = 0,
rampTime = ramp_time
)
sim.addPreTimeStepRunnable (wall_loader2)


#bounding with bottom wall#
sim.createInteractionGroup (NRotBondedWallPrms (
	name = "bottom_wall_bonds",
	wallName = "bottom_wall",
	normalK = nP,
	particleTag = 1))


# bounding with top wal
sim.createInteractionGroup (NRotBondedWallPrms (
	name = "top_wall_bonds",
	wallName = "top_wall",
	normalK = nP,
	particleTag = 1))




# --------- Savers ---------

#create a FieldSaver to store number of bonds:
sim.createFieldSaver (
InteractionScalarFieldSaverPrms(
interactionName="pp_bonds",
fieldName="count",
fileName="FieldSaver/OUT_nb.dat",
fileFormat="SUM",
beginTimeStep=0,
endTimeStep=Nsym,
timeStepIncr=fs_step
)
)

#create a FieldSaver to store the total kinetic energy of the particles:
sim.createFieldSaver (
ParticleScalarFieldSaverPrms(
fieldName="e_kin",
fileName="FieldSaver/OUT_ek.dat",
fileFormat="SUM",
beginTimeStep=0,
endTimeStep=Nsym,
timeStepIncr=fs_step
)
)

#create a FieldSaver to store potential energy stored in bonds:
sim.createFieldSaver (
InteractionScalarFieldSaverPrms(
interactionName="pp_bonds",
fieldName="potential_energy",
fileName="FieldSaver/OUT_ep.dat",
fileFormat="SUM",
beginTimeStep=0,
endTimeStep=Nsym,
timeStepIncr=fs_step
)
)

#create a FieldSaver to wall positions:
posn_saver = WallVectorFieldSaverPrms(
wallName=["bottom_wall", "top_wall"],
fieldName="Position",
fileName="FieldSaver/OUT_po.dat",
fileFormat="RAW_SERIES",
beginTimeStep=0,
endTimeStep=Nsym,
timeStepIncr=fs_step
)
sim.createFieldSaver(posn_saver)

#create a FieldSaver to wall forces:
force_saver = WallVectorFieldSaverPrms(
wallName=["bottom_wall", "top_wall"],
fieldName="Force",
fileName="FieldSaver/OUT_fo.dat",
fileFormat="RAW_SERIES",
beginTimeStep=0,
endTimeStep=Nsym,
timeStepIncr=fs_step
)
sim.createFieldSaver(force_saver)

#checkponters
sim.createCheckPointer (
      CheckPointPrms (
	     fileNamePrefix = "CP/snapshot",
	     beginTimeStep = 0,
	     endTimeStep = Nsym,
	     timeStepIncr = cp_step
       )
)


#execute the simulation:
sim.run()










